function [r,rIndiv] = JIVE_RankSelect(Datatypes,alpha,Nperm,scale, rank_eff)
%[r,rIndiv] = JIVE_RankSelect(Datatypes,alpha,Nperm,scale, rank_eff)
%Input:
%   - Datatypes: Cell array of data matrices with matching columns (Datatypes{i} gives the i'th data matrix)
%   - alpha: Significance threshold
%   - Nperm: Number of permutations
%   - scale: Should the datasets be centered and scaled by total variation? (default = 'y')
%   - rank_eff:  'Effective' rank of each matrix (default: estimated via
%   PermRankEst)
%Output:
%   - r: Rank of joint structure
%   - rIndiv: Vector of ranks for individual structure.


if(class(Datatypes)~='cell')
    disp('Error: Datatypes should be of class ''cell''');
    disp('For data matrices X1,X2,...,Xk, input Datatypes = {X1,X2,...,Xk}');
end
nDatatypes = length(Datatypes);
for i=1:nDatatypes
    [d(i),n(i)] =size(Datatypes{i});
end
n = unique(n);
if(length(n) ~= 1)
    disp('Error: Datatypes do not have same number of columns');
end

if(nargin<4)
    scale = 'y';
end

if(scale =='y')
    for(i=1:nDatatypes)
        Datatypes{i} = bsxfun(@minus,Datatypes{i},mean(Datatypes{i}')');
        Datatypes{i} = Datatypes{i}/norm(Datatypes{i}, 'fro');
    end
end

if(nargin<5)
    for i=1:nDatatypes
        rank_eff(i) = PermRankEst(Datatypes{i},alpha,Nperm);
    end
    disp('Estimated effective ranks:')
    disp(rank_eff)
end


       
%Dimension reducing transformation for high-dimensional data
for(i=1:nDatatypes)
    if(d(i) > n)
        [U,S,V] = svds(Datatypes{i},n-1);
        Datatypes{i} = S*V';
        [d(i),n] = size(Datatypes{i});
    end
end

%Estimate ranks

ScorePerm = zeros(1,Nperm);
DatatypesResid = Datatypes;
disp('Testing joint structure of rank')
for(r = 0: min(rank_eff)-1)
disp(['r = ',num2str(r)])
    [J,A] = JIVE(Datatypes,r,rank_eff-r,'n');
    for(i=1:nDatatypes)
        DatatypesResid{i} = Datatypes{i} - J((sum(d(1:(i-1)))+1):sum(d(1:i)),:);
    end
    [Jresid,Aresid] = JIVE(DatatypesResid,1,rank_eff-r-1,'n',10^(-7));
    Score = norm(Jresid+Aresid,'fro');
    DatatypesPerm =DatatypesResid;
    for(i=1:Nperm)
     for(j=1:nDatatypes)
         Datatype = DatatypesPerm{j};
         DatatypesPerm{j} = Datatype(:,randperm(n));
     end
     [Jperm,Aperm] = JIVE(DatatypesPerm,1,rank_eff-r-1,'n',10^(-7));
     ScorePerm(i) = norm(Jperm+Aperm,'fro'); 
    end
    p = (sum(ScorePerm>Score)+1)/(Nperm+1);
    if(p>alpha)
     break
    end
    if(r == min(rank_eff)-1)
        r = min(rank_eff);
    end
end
rIndiv = rank_eff-r;
return;
